@@ -27,6 +27,7 @@
 #define CARD_BUS_PATH_PREFIX "platform-"
 #define CARD_FORM_FACTOR "internal"
 #define CARD_MODEM_CLASS "modem"
+#define CARD_MODEM_NAME "Modem"
 
 struct _CadPulse
 {
@@ -58,6 +59,7 @@ typedef struct _CadPulseOperation {
 
 static void pulseaudio_cleanup(CadPulse *self);
 static gboolean pulseaudio_connect(CadPulse *self);
+static void init_pulseaudio_objects(CadPulse *self);
 
 /******************************************************************************
  * Source management
@@ -355,11 +357,19 @@ static void init_sink_info(pa_context *ctx, const pa_sink_info *info, int eol, v
 static void init_card_info(pa_context *ctx, const pa_card_info *info, int eol, void *data)
 {
     CadPulse *self = data;
+    pa_operation *op;
     const gchar *prop;
+    gboolean has_speaker = FALSE;
+    gboolean has_earpiece = FALSE;
     guint i;
 
-    if (eol != 0)
+    if (eol != 0) {
+        if (self->card_id < 0) {
+            g_critical("No suitable card found, retrying in 3s...");
+            g_timeout_add_seconds(3, G_SOURCE_FUNC(init_pulseaudio_objects), self);
+        }
         return;
+    }
 
     if (!info) {
         g_critical("PA returned no card info (eol=%d)", eol);
@@ -372,10 +382,29 @@ static void init_card_info(pa_context *ctx, const pa_card_info *info, int eol, v
     prop = pa_proplist_gets(info->proplist, PA_PROP_DEVICE_FORM_FACTOR);
     if (prop && strcmp(prop, CARD_FORM_FACTOR) != 0)
         return;
+    prop = pa_proplist_gets(info->proplist, "alsa.card_name");
+    if (prop && strcmp(prop, CARD_MODEM_NAME) == 0)
+        return;
     prop = pa_proplist_gets(info->proplist, PA_PROP_DEVICE_CLASS);
     if (prop && strcmp(prop, CARD_MODEM_CLASS) == 0)
         return;
 
+    for (i = 0; i < info->n_ports; i++) {
+        pa_card_port_info *port = info->ports[i];
+
+        if (strstr(port->name, SND_USE_CASE_DEV_SPEAKER) != NULL) {
+            has_speaker = TRUE;
+        } else if (strstr(port->name, SND_USE_CASE_DEV_EARPIECE) != NULL) {
+            has_earpiece = TRUE;
+        }
+    }
+
+    if (!has_speaker || !has_earpiece) {
+        g_message("Card '%s' lacks speaker and/or earpiece port, skipping...",
+                  info->name);
+        return;
+    }
+
     self->card_id = info->index;
 
     g_debug("CARD: idx=%u name='%s'", info->index, info->name);
@@ -390,6 +419,14 @@ static void init_card_info(pa_context *ctx, const pa_card_info *info, int eol, v
     }
 
     g_debug("CARD:   %s voice profile", self->has_voice_profile ? "has" : "doesn't have");
+
+    /* Found a suitable card, let's prepare the sink/source */
+    op = pa_context_get_sink_info_list(self->ctx, init_sink_info, self);
+    if (op)
+        pa_operation_unref(op);
+    op = pa_context_get_source_info_list(self->ctx, init_source_info, self);
+    if (op)
+        pa_operation_unref(op);
 }
 
 /******************************************************************************
@@ -434,12 +471,6 @@ static void init_card_info(pa_context *ctx, const pa_card_info *info, int eol, v
      op = pa_context_get_module_info_list(self->ctx, init_module_info, self);
      if (op)
          pa_operation_unref(op);
-     op = pa_context_get_sink_info_list(self->ctx, init_sink_info, self);
-     if (op)
-         pa_operation_unref(op);
-     op = pa_context_get_source_info_list(self->ctx, init_source_info, self);
-     if (op)
-         pa_operation_unref(op);
  }
 
 static void changed_cb(pa_context *ctx, pa_subscription_event_type_t type, uint32_t idx, void *data)
